@@ -36,45 +36,14 @@ get_sql_report_data_query <- function (key_value = NULL, name = NULL,
                                        month = NULL, day = NULL, hour = NULL, minute = NULL,
                                        interval = NULL, simulation_days = NULL, day_type = NULL,
                                        environment_name = NULL) {
-    # helper {{{
-    sep <- function (x, ignore_case = TRUE) {
-        if (is.character(x)) {
-            if (ignore_case) {
-                stri_trans_tolower(paste(paste0("\"", x, "\""), sep = ",", collapse = ","))
-            } else {
-                paste(paste0("\"", x, "\""), sep = ",", collapse = ",")
-            }
-        } else {
-            paste(x, sep = ",", collapse = ",")
-        }
-    }
-    make <- function (arg, assertion = NULL, sql_col = NULL, ignore_case = TRUE) {
-        a <- substitute(assertion)
-        if (is.null(arg)) return(NULL)
-        eval(a)
-        if (is.null(sql_col)) {
-            sql_col <- stri_trans_totitle(deparse(substitute(arg)))
-        }
-        if (is.character(arg) && ignore_case) {
-            paste0("lower(", sql_col, ") IN (", sep(unique(arg), TRUE), ")")
-        } else {
-            paste0(sql_col, " IN (", sep(unique(arg), FALSE), ")")
-        }
-    }
-    `%and%` <- function (x, y) {
-        if (is.null(y)) return(x)
-        if (is.null(x)) return(y)
-        paste0("(", x, ") AND (", y, ")")
-    }
-    # }}}
     # Query for Time {{{
     time <- NULL %and%
-        make(month, assert(are_count(month), month <= 12L)) %and%
-        make(day, assert(are_count(day), day <= 31L)) %and%
-        make(hour, assert(are_count(hour, TRUE), hour <= 23L)) %and%
-        make(minute, assert(are_count(minute, TRUE), minute <= 59L)) %and%
-        make(interval, assert(are_count(interval))) %and%
-        make(simulation_days, assert(are_count(simulation_days)), "SimulationDays")
+        .sql_make(month, assert(are_count(month), month <= 12L), "Month") %and%
+        .sql_make(day, assert(are_count(day), day <= 31L), "Day") %and%
+        .sql_make(hour, assert(are_count(hour, TRUE), hour <= 23L), "Hour") %and%
+        .sql_make(minute, assert(are_count(minute, TRUE), minute <= 59L), "Minute") %and%
+        .sql_make(interval, assert(are_count(interval)), "Interval") %and%
+        .sql_make(simulation_days, assert(are_count(simulation_days)), "SimulationDays")
 
     if (!is.null(day_type)) {
         dt <- match_in_vec(day_type, label = TRUE,
@@ -84,7 +53,7 @@ get_sql_report_data_query <- function (key_value = NULL, name = NULL,
             )
         )
         assert(!is.na(dt), msg = paste0("Invalid day type found: ", collapse(day_type[is.na(dt)]), "."))
-        time <- time %and% make(dt, sql_col = "DayType")
+        time <- time %and% .sql_make(dt, sql_col = "DayType")
     }
 
     if (is.null(environment_name)) {
@@ -95,7 +64,7 @@ get_sql_report_data_query <- function (key_value = NULL, name = NULL,
             (
             SELECT *
             FROM EnvironmentPeriods
-            WHERE ", make(environment_name, sql_col = "EnvironmentName"), "
+            WHERE ", .sql_make(environment_name, sql_col = "EnvironmentName"), "
             )"
         )
     }
@@ -142,7 +111,7 @@ get_sql_report_data_query <- function (key_value = NULL, name = NULL,
                 lubridate::minute(p),
                 sep = "-"
             )
-            period <- make(period, sql_col = "DateTime")
+            period <- .sql_make(period, sql_col = "DateTime")
 
             dt <- "(Month || \"-\" || Day || \"-\" || Hour || \"-\" || Minute) AS DateTime"
             time <- paste0("
@@ -171,8 +140,8 @@ get_sql_report_data_query <- function (key_value = NULL, name = NULL,
     }
 
     key_var <- NULL %and%
-        make(key_value, assert(is.character(key_value), no_na(key_value)), "KeyValue") %and%
-        make(name, assert(is.character(name), no_na(name)), "Name")
+        .sql_make(key_value, assert(is.character(key_value), no_na(key_value)), "KeyValue") %and%
+        .sql_make(name, assert(is.character(name), no_na(name)), "Name")
 
     if (is.null(key_var)) {
         key_var <- "ReportDataDictionary"
@@ -222,35 +191,6 @@ get_sql_report_data_query <- function (key_value = NULL, name = NULL,
 get_sql_tabular_data_query <- function (report_name = NULL, report_for = NULL,
                                         table_name = NULL, column_name = NULL,
                                         row_name = NULL) {
-    # helper {{{
-    sep <- function (x, ignore_case = TRUE) {
-        if (is.character(x)) {
-            if (ignore_case) {
-                stri_trans_tolower(paste(paste0("\"", x, "\""), sep = ",", collapse = ","))
-            } else {
-                paste(paste0("\"", x, "\""), sep = ",", collapse = ",")
-            }
-        } else {
-            paste(x, sep = ",", collapse = ",")
-        }
-    }
-    make <- function (arg, assertion = NULL, sql_col = NULL, ignore_case = TRUE) {
-        a <- substitute(assertion)
-        if (is.null(arg)) return(NULL)
-        eval(a)
-        sql_col <- deparse(substitute(arg))
-        if (is.character(arg) && ignore_case) {
-            paste0("lower(", sql_col, ") IN (", sep(unique(arg), TRUE), ")")
-        } else {
-            paste0(sql_col, " IN (", sep(unique(arg), FALSE), ")")
-        }
-    }
-    `%and%` <- function (x, y) {
-        if (is.null(y)) return(x)
-        if (is.null(x)) return(y)
-        paste0("(", x, ") AND (", y, ")")
-    }
-    # }}}
     # basic view {{{
     view <-
         "
@@ -286,11 +226,11 @@ get_sql_tabular_data_query <- function (report_name = NULL, report_for = NULL,
     # }}}
 
     q <- NULL %and%
-        make(report_name, assert(is.character(report_name), no_na(report_name))) %and%
-        make(report_for, assert(is.character(report_for), no_na(report_for))) %and%
-        make(table_name, assert(is.character(table_name), no_na(table_name))) %and%
-        make(column_name, assert(is.character(column_name), no_na(column_name))) %and%
-        make(row_name, assert(is.character(row_name), no_na(row_name)))
+        .sql_make(report_name, assert(is.character(report_name), no_na(report_name))) %and%
+        .sql_make(report_for, assert(is.character(report_for), no_na(report_for))) %and%
+        .sql_make(table_name, assert(is.character(table_name), no_na(table_name))) %and%
+        .sql_make(column_name, assert(is.character(column_name), no_na(column_name))) %and%
+        .sql_make(row_name, assert(is.character(row_name), no_na(row_name)))
 
     if (is.null(q)) return(view)
 
@@ -305,7 +245,7 @@ list_sql_table <- function (sql) {
 # }}}
 # get_sql_report_data {{{
 get_sql_report_data <- function (sql, key_value = NULL, name = NULL, year = NULL,
-                                 tz = "UTC", case = "auto", all = FALSE,
+                                 tz = "UTC", case = "auto", all = FALSE, wide = FALSE,
                                  period = NULL, month = NULL, day = NULL, hour = NULL, minute = NULL,
                                  interval = NULL, simulation_days = NULL, day_type = NULL,
                                  environment_name = NULL) {
@@ -329,21 +269,26 @@ get_sql_report_data <- function (sql, key_value = NULL, name = NULL, year = NULL
         # get wday per environment
         w <- get_sql_wday(sql)
 
-        # for WinterDesignDay and SummerDesignDay, set to Monday
-        set(w, NULL, "date", lubridate::make_date(year, w$month, w$day))
-        set(w, NULL, "dt", get_epw_wday(w$day_type))
-        w[day_type %chin% c("WinterDesignDay", "SummerDesignDay"), dt := 1L]
+        # in case there is no valid day type and get_sql_wday() returns nothing
+        if (!nrow(w)) {
+            set(res, NULL, "year", year)
+        } else {
+            # for WinterDesignDay and SummerDesignDay, set to Monday
+            set(w, NULL, "date", lubridate::make_date(year, w$month, w$day))
+            set(w, NULL, "dt", get_epw_wday(w$day_type))
+            w[day_type %chin% c("WinterDesignDay", "SummerDesignDay"), dt := 1L]
 
-        if (any(!is.na(w$dt))) {
-            for (i in which(!is.na(w$dt))) {
-                set(w, i, "year", find_nearst_wday_year(w$date[i], w$dt[i], year, leap))
+            if (any(!is.na(w$dt))) {
+                for (i in which(!is.na(w$dt))) {
+                    set(w, i, "year", find_nearst_wday_year(w$date[i], w$dt[i], year, leap))
+                }
             }
+
+            # make sure all environments have a year value
+            w[is.na(dt), year := lubridate::year(Sys.Date())]
+
+            set(res, NULL, "year", w[J(res$environment_period_index), on = "environment_period_index", year])
         }
-
-        # make sure all environments have a year value
-        w[is.na(dt), year := lubridate::year(Sys.Date())]
-
-        set(res, NULL, "year", w[J(res$environment_period_index), on = "environment_period_index", year])
     } else {
         set(res, NULL, "year", year)
     }
@@ -352,28 +297,31 @@ get_sql_report_data <- function (sql, key_value = NULL, name = NULL, year = NULL
         lubridate::make_datetime(res$year, res$month, res$day, res$hour, res$minute, tz = tz)
     )
 
-    set(res, NULL, c("year", "environment_period_index"), NULL)
+    set(res, NULL, "year", NULL)
 
-    # stop if any invalid datetime found
+    # warning if any invalid datetime found
     # month, day, hour, minute may be NA if reporting frequency is Monthly or
     # RunPeriod
     if (anyNA(res[!is.na(month) & !is.na(day) & !is.na(hour) & !is.na(minute), datetime])) {
         invld <- res[!is.na(month) & !is.na(day) & !is.na(hour) & !is.na(minute)]
         mes <- invld[, paste0("Original: ", month, "-", day, " ",  hour, ":", minute,
             " --> New year: ", year)]
-        abort("error_invalid_epw_date_introduced",
+        warn("warn_invalid_epw_date_introduced",
             paste0("Invalid date introduced with input start year:\n",
                 paste0(mes, collapse = "\n")
             )
         )
     }
 
-    if (!all) {
+    if (!all & !wide) {
         res <- res[, .SD, .SDcols = c("datetime", "key_value", "name", "units", "value")]
     } else {
         setcolorder(res, c("datetime", setdiff(names(res), "datetime")))
     }
 
+    # change to wide table
+    if (wide) res <- report_dt_to_wide(res, all)
+
     if (not_empty(case)) {
         assert(is_scalar(case))
         case_name <- as.character(case)
@@ -391,9 +339,77 @@ get_sql_report_data_dict <- function (sql) {
 # }}}
 # get_sql_tabular_data {{{
 get_sql_tabular_data <- function (sql, report_name = NULL, report_for = NULL,
-                                  table_name = NULL, column_name = NULL, row_name = NULL) {
+                                  table_name = NULL, column_name = NULL, row_name = NULL,
+                                  case = "auto", wide = FALSE, string_value = !wide) {
     q <- get_sql_tabular_data_query(report_name, report_for, table_name, column_name, row_name)
-    setnames(get_sql_query(sql, q), "tabular_data_index", "index")[]
+    dt <- setnames(get_sql_query(sql, q), "tabular_data_index", "index")[]
+
+    if (not_empty(case)) {
+        assert(is_scalar(case))
+        case_name <- as.character(case)
+        set(dt, NULL, "case", case_name)
+        setcolorder(dt, c("case", setdiff(names(dt), "case")))
+    }
+
+    if (!wide) return(dt)
+
+    if (!string_value) {
+        dt[!J(c("", " ")), on = "units", is_num := TRUE]
+        # https://stackoverflow.com/questions/638565/parsing-scientific-notation-sensibly
+        dt[, is_num := any(stri_detect_regex(value, "^\\s*-?\\d+(?:\\.\\d*)?(?:[eE][+\\-]?\\d+)?$")),
+            by = c("report_name", "report_for", "table_name", "column_name")]
+    }
+
+    # add row index
+    dt[, row_index := seq_len(.N), by = c("case"[has_name(dt, "case")], "report_name", "report_for", "table_name", "column_name")]
+
+    # remove empty rows
+    dt <- dt[!J(c("", "-"), c("", "-")), on = c("row_name", "value")]
+
+    # fill downwards row names
+    if (nrow(dt)) {
+        dt[, row_name := row_name[[1L]],
+            by = list(report_name, report_for, table_name, column_name, cumsum(row_name != ""))]
+    }
+
+    # combine column names and units
+    dt[!J("", " "), on = "units", column_name := paste0(column_name, " [", units, "]")]
+
+    l <- split(dt, by = c("report_name", "report_for", "table_name"))
+    lapply(l, wide_tabular_data, string_value = string_value)
+}
+# }}}
+# wide_tabular_data {{{
+wide_tabular_data <- function (dt, string_value = TRUE) {
+    # retain original column order
+    cols <- unique(dt$column_name)
+
+    # get numeric columns
+    cols_num <- unique(dt$column_name[dt$is_num])
+
+    # format table
+    if (has_name(dt, "case")) {
+        dt <- data.table::dcast.data.table(dt,
+            case + report_name + report_for + table_name + row_index + row_name ~ column_name,
+            value.var = "value"
+        )
+    } else {
+        dt <- data.table::dcast.data.table(dt,
+            report_name + report_for + table_name + row_index + row_name ~ column_name,
+            value.var = "value"
+        )
+    }
+
+    # clean
+    data.table::set(dt, NULL, "row_index", NULL)
+
+    # coerece type
+    if (!string_value && length(cols_num)) {
+        as_numeric <- function (x) suppressWarnings(as.numeric(x))
+        dt[, c(cols_num) := lapply(.SD, as_numeric), .SDcols = cols_num]
+    }
+
+    dt[]
 }
 # }}}
 # get_sql_wday {{{
@@ -410,8 +426,143 @@ get_sql_wday <- function (sql) {
     get_sql_query(sql, q)
 }
 # }}}
+# get_sql_date {{{
+get_sql_date <- function (sql, environment_period_index, simulation_days) {
+    cond <- NULL %and%
+        .sql_make(environment_period_index, sql_col = "EnvironmentPeriodIndex") %and%
+        .sql_make(simulation_days, sql_col = "SimulationDays")
+    q <- paste0("
+         SELECT DISTINCT
+                EnvironmentPeriodIndex as environment_period_index,
+                SimulationDays as simulation_days,
+                Month AS month,
+                Day AS day
+         FROM Time
+         WHERE ", cond, " AND (Month IS NOT NULL) AND (Day IS NOT NULL)"
+        )
+    get_sql_query(sql, q)
+}
+# }}}
 # tidy_sql_name {{{
 tidy_sql_name <- function (x) {
     setnames(x, stri_sub(gsub("([A-Z])", "_\\L\\1", names(x), perl = TRUE), 2L))
 }
 # }}}
+# report_dt_to_wide {{{
+report_dt_to_wide <- function (dt, date_components = FALSE) {
+    assert(has_name(dt, c("datetime", "month", "day", "hour", "minute",
+        "key_value", "name", "environment_period_index", "environment_name",
+        "reporting_frequency", "is_meter", "simulation_days", "day_type"
+    )))
+
+    # change detailed level frequency to "Each Call"
+    dt[, Variable := reporting_frequency]
+    dt[J("HVAC System Timestep"), on = "reporting_frequency", Variable := "Each Call"]
+    dt[J("Zone Timestep"), on = "reporting_frequency", Variable := "TimeStep"]
+    # combine key_value, name, and unit
+    dt[J(1L), on = "is_meter", Variable := paste0(name, " [", units, "](", Variable, ")")]
+    dt[J(0L), on = "is_meter", Variable := paste0(key_value, ":", name, " [", units, "](", Variable, ")")]
+
+    # handle RunPeriod frequency
+    if ("Run Period" %in% unique(dt$reporting_frequency)) {
+        last_day <- dt[!is.na(datetime), .SD[.N],
+            .SDcols = c("datetime", "month", "day", "hour", "minute"),
+            by = "environment_period_index"
+        ]
+        set(last_day, NULL, "reporting_frequency", "Run Period")
+
+        dt[last_day, on = c("environment_period_index", "reporting_frequency"),
+            `:=`(datetime = i.datetime, month = i.month, day = i.day,
+                 hour = i.hour, minute = i.minute
+            )
+        ]
+    }
+
+    # format datetime
+    dt[, `Date/Time` := paste0(" ",
+        stringi::stri_pad(month, 2, pad = "0"), "/",
+        stringi::stri_pad(day, 2, pad = "0"), "  ",
+        stringi::stri_pad(hour, 2, pad = "0"), ":",
+        stringi::stri_pad(minute, 2, pad = "0")
+    )]
+
+    # handle special cases
+    if (nrow(dt) & all(is.na(dt$datetime))) {
+        dt[reporting_frequency != "Monthly", `Date/Time` := paste0("simdays=", simulation_days)]
+        dt[reporting_frequency == "Monthly", `Date/Time` := {
+            m <- get_sql_date(sql, .BY$environment_period_index, simulation_days)$month
+            get_epw_month(m, label = TRUE)
+        }, by = "environment_period_index"]
+    }
+
+    if (date_components) {
+        # fill day_type
+        dt[is.na(day_type) & !is.na(datetime) & hour == 24L,
+            `:=`(day_type = wday(datetime - hours(1L), label = TRUE))
+        ]
+        dt[is.na(day_type) & !is.na(datetime) & hour != 24L,
+            `:=`(day_type = wday(datetime, label = TRUE))
+        ]
+
+        if (has_name(dt, "case")) {
+            dt <- dcast.data.table(dt, case +
+                environment_period_index + environment_name + simulation_days +
+                datetime + month + day + hour + minute +
+                day_type + `Date/Time` ~ Variable,
+                value.var = "value")
+        } else {
+            dt <- dcast.data.table(dt,
+                environment_period_index + environment_name + simulation_days +
+                datetime + month + day + hour + minute +
+                day_type + `Date/Time` ~ Variable,
+                value.var = "value")
+        }
+    } else {
+        if (has_name(dt, "case")) {
+            dt <- dcast.data.table(dt, case +
+                environment_period_index + environment_name + simulation_days +
+                `Date/Time` ~ Variable,
+                value.var = "value")[, .SD, .SDcols = -(1:4)]
+        } else {
+            dt <- dcast.data.table(dt,
+                environment_period_index + environment_name + simulation_days +
+                `Date/Time` ~ Variable,
+                value.var = "value")[, .SD, .SDcols = -(1:3)]
+        }
+    }
+
+    dt
+}
+# }}}
+
+# helper {{{
+.sql_sep <- function (x, ignore_case = TRUE) {
+    if (is.character(x)) {
+        if (ignore_case) {
+            stri_trans_tolower(paste(paste0("\"", x, "\""), sep = ",", collapse = ","))
+        } else {
+            paste(paste0("\"", x, "\""), sep = ",", collapse = ",")
+        }
+    } else {
+        paste(x, sep = ",", collapse = ",")
+    }
+}
+.sql_make <- function (arg, assertion = NULL, sql_col = NULL, ignore_case = TRUE, env = parent.frame()) {
+    a <- substitute(assertion, env)
+    if (is.null(arg)) return(NULL)
+    eval(a)
+    if (is.null(sql_col)) {
+        sql_col <- deparse(substitute(arg))
+    }
+    if (is.character(arg) && ignore_case) {
+        paste0("lower(", sql_col, ") IN (", .sql_sep(unique(arg), TRUE), ")")
+    } else {
+        paste0(sql_col, " IN (", .sql_sep(unique(arg), FALSE), ")")
+    }
+}
+`%and%` <- function (x, y) {
+    if (is.null(y)) return(x)
+    if (is.null(x)) return(y)
+    paste0("(", x, ") AND (", y, ")")
+}
+# }}}
