@@ -2,7 +2,6 @@
 #' @importFrom cli cat_line
 #' @importFrom crayon red
 #' @importFrom data.table data.table setattr setnames
-#' @importFrom later later
 #' @importFrom lubridate hms with_tz
 #' @importFrom tools file_path_sans_ext
 #' @importFrom processx process
@@ -26,7 +25,20 @@ NULL
 #'
 #' @examples
 #' \dontrun{
-#' clean_wd("foo.idf")
+#' # run a test simulation
+#' idf_path <- system.file("extdata/1ZoneUncontrolled.idf", package = "eplusr")
+#' epw_path <- file.path(eplus_config(8.8)$dir, "WeatherData",
+#'      "USA_CA_San.Francisco.Intl.AP.724940_TMY3.epw"
+#' )
+#' dir <- file.path(tempdir(), "test")
+#' run_idf(idf_path, epw_path, output_dir = dir, echo = FALSE)
+#'
+#' list.files(dir)
+#'
+#' # remove all output files
+#' clean_wd(file.path(dir, basename(idf_path)))
+#'
+#' list.files(dir)
 #' }
 #' @export
 #' @author Hongyuan Jia
@@ -92,9 +104,9 @@ clean_wd <- function (path) {
 #' @param annual Force design-day-only simulation. Default: `FALSE`.
 #' @param expand_obj Whether to run ExpandObject preprocessor before simulation.
 #'     Default: `TRUE`.
-#' @param echo Whether to show standard output and error from EnergyPlus
-#'     command line interface for `run_idf()` and simulation status for
-#'     `run_multi()`.Default: `TRUE`.
+#' @param echo Only applicable when `wait` is `TRUE`. Whether to show standard
+#'     output and error from EnergyPlus command line interface for `run_idf()`
+#'     and simulation status for `run_multi()`.Default: `TRUE`.
 #' @param wait If `TRUE`, R will hang on and wait all EnergyPlus simulations
 #'     finish. If `FALSE`, all EnergyPlus simulations are run in the background.
 #'     and a [processx::process] object is returned. Note that, if `FALSE`, R is
@@ -107,10 +119,9 @@ clean_wd <- function (path) {
 #'
 #' @details
 #'
-#' [later](https://cran.r-project.org/package=later) package is used to poll the
-#'     standard output and error of background EnergyPlus process or background
-#'     R process that handles parallel simulations. The print interval is set to
-#'     0.5 sec.
+#' `run_idf()` and `run_multi()` currently only support EnergyPlus v8.3 and
+#'     above. This is because eplusr uses EnergyPlus command line interface
+#'     which is a new feature as of EnergyPlus v8.3.
 #'
 #' For `run_idf()`, a named list will be returned:
 #'
@@ -139,13 +150,10 @@ clean_wd <- function (path) {
 #' `process`) with same column names as above, and also another two columns:
 #'
 #'   * `index`: The index of simulation
-#'   * `status`: The status of simulation status. Should be one of below:
-#'       - `"completed"`: the simulation job is completed. This only indicates
-#'          that the calling of EnergyPlus was successfully and EnergyPlus was
-#'          not terminated during simulation. Even `"completed" is `TRUE`, the
-#'          job can still end with error.  Please check `exit_status` to
-#'          determine whether EnergyPlus ran successfully
-#'       - `"terminated"`: the simulation job started but was terminated
+#'   * `status`: The status of simulation. Should be one of below:
+#'       - `"completed"`: the simulation job is completed successfully.
+#'       - `"failed"`: the simulation job ended with error.
+#'       - `"terminated"`: the simulation job started but was terminated.
 #'       - `"cancelled"`: the simulation job was cancelled, i.e. did not start
 #'          at all.
 #'
@@ -197,7 +205,7 @@ run_idf <- function (model, weather, output_dir, design_day = FALSE,
                      annual = FALSE, expand_obj = TRUE, wait = TRUE, echo = TRUE, eplus = NULL) {
 
     model <- normalizePath(model, mustWork = TRUE)
-    weather <- normalizePath(weather, mustWork = TRUE)
+    if (!is.null(weather)) weather <- normalizePath(weather, mustWork = TRUE)
 
     eplus <- eplus %||% as.character(get_idf_ver(read_lines(model)))
     energyplus_exe <- eplus_exe(eplus)
@@ -221,7 +229,11 @@ run_idf <- function (model, weather, output_dir, design_day = FALSE,
 
     # copy input files
     loc_m <- copy_run_files(model, output_dir)
-    loc_w <- copy_run_files(weather, output_dir)
+    if (is.null(weather)) {
+        loc_w <- NULL
+    } else {
+        loc_w <- copy_run_files(weather, output_dir)
+    }
 
     # clean wd
     clean_wd(loc_m)
@@ -231,7 +243,8 @@ run_idf <- function (model, weather, output_dir, design_day = FALSE,
         wait = wait, echo = echo)
 
     res$idf <- model
-    res$epw <- weather
+    # in case no weather is given
+    res["epw"] <- list(weather)
     res$version <- as.character(eplus_config(eplus)$version)
 
     res[c("idf", "epw", "exit_status", "start_time", "end_time", "output_dir",
@@ -244,23 +257,43 @@ run_idf <- function (model, weather, output_dir, design_day = FALSE,
 # run_multi {{{
 run_multi <- function (model, weather, output_dir, design_day = FALSE,
                        annual = FALSE, wait = TRUE, echo = TRUE, eplus = NULL) {
-    assert(is_flag(design_day))
-    assert(is_flag(annual))
     assert(is_flag(wait))
     assert(is_flag(echo))
 
-    if (annual && design_day) {
-        stop("Cannot force both design-day and annual simulations.", call. = FALSE)
+    if (!is_scalar(model)) {
+        if (!is.null(weather) && !is_scalar(weather)) {
+            assert(have_same_len(model, weather))
+        }
+        if (!is.null(eplus) && !is_scalar(eplus)) {
+            assert(have_same_len(model, eplus))
+        }
+        if (!is_scalar(design_day)) {
+            assert(have_same_len(model, design_day))
+            assert(is.logical(design_day), no_na(design_day))
+        }
+        if (!is_scalar(annual)) {
+            assert(have_same_len(model, annual))
+            assert(is.logical(annual), no_na(annual))
+        }
     }
 
-    if (!is_scalar(model) && !is_scalar(weather))
-        assert(have_same_len(model, weather))
-
-    if (!is_scalar(model) && !is.null(eplus) && !is_scalar(eplus))
-        assert(have_same_len(model, eplus))
+    if (any(annual & design_day)) {
+        abort("error_run_both_ddy_annual", "Cannot force both design-day and annual simulations.")
+    }
 
     model <- normalizePath(model, mustWork = TRUE)
-    weather <- normalizePath(weather, mustWork = TRUE)
+
+    if (is.null(weather)) {
+        input_weather <- rep(NA_character_, length(model))
+        weather <- list(NULL)
+    } else {
+        if (length(weather) == 1L) weather <- rep(weather, length(model))
+        input_weather <- vcapply(weather,
+            function (x) if (is.null(x)) NA_character_ else normalizePath(x, mustWork = TRUE)
+        )
+        weather <- as.list(input_weather)
+        weather[is.na(input_weather)] <- list(NULL)
+    }
 
     if (is.null(eplus)) {
         ver_list <- lapply(model, function (x) {
@@ -270,8 +303,10 @@ run_multi <- function (model, weather, output_dir, design_day = FALSE,
         if (any(ver_miss)) {
             msg <- paste0("  ", seq_along(model)[ver_miss], "| ", surround(model[ver_miss]),
                 collapse = "\n")
-            stop("Missing version field in input IDF file. Failed to determine the ",
-                "version of EnergyPlus to use:\n", msg, call. = FALSE)
+            abort("error_miss_idf_ver", paste0(
+                "Missing version field in input IDF file. Failed to determine the ",
+                "version of EnergyPlus to use:\n", msg
+            ))
         }
 
         eplus <- unlist(ver_list)
@@ -279,9 +314,11 @@ run_multi <- function (model, weather, output_dir, design_day = FALSE,
 
     energyplus_exe <- vapply(eplus, eplus_exe, FUN.VALUE = character(1))
 
-    if (anyDuplicated(model) & is.null(output_dir))
-        stop("`model` cannot have any duplications when `output_dir` is NULL.",
-            call. = FALSE)
+    if (anyDuplicated(model) & is.null(output_dir)) {
+        abort("error_run_duplicated_model",
+            "`model` cannot have any duplications when `output_dir` is NULL."
+        )
+    }
 
     if (is.null(output_dir)) {
         output_dir <- dirname(model)
@@ -294,20 +331,38 @@ run_multi <- function (model, weather, output_dir, design_day = FALSE,
     jobs <- data.table::data.table(input_model = model, output_dir = output_dir)
 
     if (anyDuplicated(jobs))
-        stop("Duplication found in the combination of `model` and `output_dir`.",
+        abort("error_run_duplicated_job", paste0(
+            "Duplication found in the combination of `model` and `output_dir`.",
             " One model could not be run in the same output directory multiple ",
-            "times simultaneously.", call. = FALSE)
+            "times simultaneously."
+        ))
 
-    d <- output_dir[!dir.exists(output_dir)]
+    d <- unique(output_dir[!dir.exists(output_dir)])
     created <- vapply(d, dir.create, logical(1L), showWarnings = FALSE, recursive = TRUE)
-    if (any(!created))
-        stop("Failed to create output directory:\n",
-            paste0(surround(d[!created]), collapse = "\n"), call. = FALSE)
+    if (any(!created)) {
+        abort("error_create_output_dir", paste0("Failed to create output directory:\n",
+            paste0(surround(d[!created]), collapse = "\n")
+        ))
+    }
 
-    jobs[, `:=`(input_weather = weather, energyplus = energyplus_exe)]
-    jobs[, `:=`(model = copy_run_files(model, output_dir),
-                 weather = copy_run_files(weather, output_dir),
-                 index = .I, annual = annual, design_day = design_day)]
+    jobs[, `:=`(
+        energyplus = energyplus_exe,
+        model = copy_run_files(model, output_dir),
+        index = .I, annual = annual, design_day = design_day
+    )]
+    if (is.null(weather)) {
+        jobs[, `:=`(input_weather = NA_character_, weather = list(NULL))]
+    } else {
+        if (any(!is.na(input_weather))) {
+            weather[!is.na(input_weather)] <- as.list(
+                copy_run_files(
+                    unlist(weather[!is.na(input_weather)]),
+                    output_dir[!is.na(input_weather)]
+                )
+            )
+        }
+        jobs[, `:=`(input_weather = input_weather, weather = weather)]
+    }
 
     options <- list(num_parallel = eplusr_option("num_parallel"), echo = echo)
 
@@ -331,17 +386,6 @@ run_multi <- function (model, weather, output_dir, design_day = FALSE,
             run_parallel_jobs(jobs, options)
         }, args = list(ext_funs = ext_funs, jobs = jobs, options = options))
 
-        # watch_proc {{{
-        # reference: https://yihui.name/en/2017/10/later-recursion
-        watch_proc = function() {
-            if (proc_print(proc, c(TRUE, TRUE))) {
-                later::later(watch_proc, 0.5)
-            }
-        }
-        # }}}
-
-        if (echo) watch_proc()
-
         proc
     }
 
@@ -382,13 +426,11 @@ run_parallel_jobs <- function(jobs, options) {
     if (nrow(jobs) == 0) return()
     assert(is_integer(options$num_parallel))
 
-    start_time <- Sys.time()
-
     ## Kill all child processes if we quit from this function
     on.exit(kill_jobs(jobs, options), add = TRUE)
 
     # initialize job status and worker
-    jobs[, `:=`(status = "waiting", index_str = lpad(index), process = list(),
+    jobs[, `:=`(status = "waiting", index_str = lpad(index, "0"), process = list(),
         stdout = list(), stderr = list(), exit_status = NA_integer_
     )]
 
@@ -421,13 +463,6 @@ run_parallel_jobs <- function(jobs, options) {
     jobs[, .SD, .SDcols = c("index", "status", "idf", "epw", "exit_status",
         "start_time", "end_time", "energyplus", "output_dir", "stdout", "stderr"
     )]
-
-    end_time <- Sys.time()
-
-    data.table::setattr(jobs, "start_time", start_time)
-    data.table::setattr(jobs, "end_time", end_time)
-
-    jobs
 }
 # }}}
 # kill_jobs {{{
@@ -441,8 +476,8 @@ kill_jobs <- function(jobs, options) {
 
     if (any(jobs$status == "terminated")) {
         jobs[status == "terminated", `:=`(
-            stdout = lapply(process, function (x) x$read_all_output_lines()),
-            stderr = lapply(process, function (x) x$read_all_error_lines()),
+            stdout = lapply(process, function (x) tryCatch(x$read_all_output_lines(), error = function (e) NA_character_)),
+            stderr = lapply(process, function (x) tryCatch(x$read_all_error_lines(), error = function (e) NA_character_)),
             exit_status = vapply(process, function (x) x$get_exit_status(), integer(1))
         )]
     }
@@ -490,11 +525,10 @@ run_job <- function(jobs, options, progress_bar) {
 
     jobs[status == "ready", `:=`(status = "newly_started",
         process = list(energyplus(eplus = energyplus, model = model,
-            weather = weather, output_dir = output_dir, annual = annual,
+            weather = unlist(weather), output_dir = output_dir, annual = annual,
             design_day = design_day, wait = FALSE, echo = FALSE)$process)
     )]
 
-    data.table::data.table(x = 1:5)[, c("y", "z") := list("a", "b")][]
     if (any(jobs$status == "newly_started")) {
         completed <- jobs[status == "newly_started",
             sim_status("run", index_str, model, weather)
@@ -507,7 +541,9 @@ run_job <- function(jobs, options, progress_bar) {
     }
 
     jobs[status == "newly_started", `:=`(status = "running",
-        start_time = do.call("c", lapply(process, function (x) x$get_start_time()))
+        start_time = do.call("c", lapply(process,
+            function (x) lubridate::with_tz(x$get_start_time(), Sys.timezone())
+        ))
     )]
 
     jobs
@@ -515,17 +551,17 @@ run_job <- function(jobs, options, progress_bar) {
 # }}}
 # are_all_completed {{{
 are_all_completed <- function(jobs) {
-    all(jobs$status == "completed")
+    all(jobs$status %chin% c("completed", "failed"))
 }
 # }}}
 # handle_events {{{
 handle_events <- function(jobs, options, progress_bar) {
     jobs[status == "running" &
-         vapply(process, function (x) {!is.null(x) && !x$is_alive()}, logical(1)),
+         vlapply(process, function (x) !is.null(x) && !x$is_alive()),
         `:=`(
             stdout = lapply(process, function (x) x$read_all_output_lines()),
             stderr = lapply(process, function (x) x$read_all_error_lines()),
-            exit_status = vapply(process, function (x) x$get_exit_status(), integer(1)),
+            exit_status = viapply(process, function (x) x$get_exit_status()),
             status = "newly_completed", end_time = Sys.time()
         )
     ]
@@ -534,7 +570,7 @@ handle_events <- function(jobs, options, progress_bar) {
         num <- sum(jobs$status == "newly_completed")
 
         completed <- jobs[status == "newly_completed",
-            sim_status("complete", index_str, model, weather)
+            sim_status("complete", index_str, model, weather, exit_status)
         ]
 
         if (options$echo) {
@@ -543,28 +579,35 @@ handle_events <- function(jobs, options, progress_bar) {
         progress_bar$tick(num)
     }
 
-    jobs[status == "newly_completed", `:=`(status = "completed")]
+    jobs[status == "newly_completed", `:=`(status = ifelse(exit_status == 0L, "completed", "failed"))]
 
     jobs
 }
 # }}}
 # sim_status {{{
-sim_status <- function (type, index, model, weather) {
+sim_status <- function (type, index, model, weather, exit_code = NULL) {
     status <- c("run", "complete", "cancel", "terminate")
-    if (type %in% status) {
+    if (length(type) ==1L && type %in% status) {
         type <- switch(type,
             run       = "RUNNING   ",
             complete  = "COMPLETED ",
             cancel    = "CANCELLED ",
             terminate = "TERMINATED"
         )
+        if (!is.null(exit_code)) type[exit_code != 0L] <- "FAILED    "
     }
 
-    paste0(lpad(index), "|", type, " --> ",
-        "[IDF]", surround(basename(model)),
-        " + ",
-        "[EPW]", surround(basename(weather))
+    mes <- paste0(lpad(index, "0"), "|", type, " --> ",
+        "[IDF]", surround(basename(model))
     )
+
+    has_epw <- !vlapply(weather, function (x) is.null(x) || is.na(x))
+
+    if (any(has_epw)) {
+        mes[has_epw] <- paste0(mes, " + ", "[EPW]", surround(basename(unlist(weather[has_epw]))))
+    }
+
+    mes
 }
 # }}}
 # energyplus {{{
@@ -578,7 +621,7 @@ energyplus <- function (eplus, model, weather, output_dir, output_prefix = NULL,
     assert(
         file.exists(eplus),
         file.exists(model),
-        file.exists(weather),
+        is.null(weather) || file.exists(weather),
         is.null(output_dir) || dir.exists(output_dir),
         is.null(output_prefix) || is_string(output_prefix),
         is_flag(expand_obj),
@@ -669,8 +712,9 @@ energyplus <- function (eplus, model, weather, output_dir, output_prefix = NULL,
     if (!is.null(idd)) cmd_idd <- paste0("--idd", shQuote(idd)) else cmd_idd <- NULL
     # }}}
 
+    arg_weather <- if (is.null(weather)) NULL else c("--weather", weather)
     args <- c(
-        "--weather", weather,
+        arg_weather,
         "--output-directory", output_dir,
         "--output-prefix", output_prefix,
         "--output-suffix", output_suffix,
@@ -687,7 +731,7 @@ energyplus <- function (eplus, model, weather, output_dir, output_prefix = NULL,
 
     # run energyplus
     proc <- processx::process$new(command = eplus, args = args, wd = dirname(model),
-        cleanup = TRUE, windows_verbatim_args = TRUE,
+        cleanup = TRUE, windows_verbatim_args = FALSE,
         stdout = "|", stderr = "|", post_process = function () Sys.time()
     )
 
@@ -709,16 +753,6 @@ energyplus <- function (eplus, model, weather, output_dir, output_prefix = NULL,
 
         res <- eplus_run_wait(proc, echo = echo)
     } else {
-        # watch_proc {{{
-        # reference: https://yihui.name/en/2017/10/later-recursion
-        watch_proc = function() {
-            if (proc_print(proc, c(TRUE, TRUE))) {
-                later::later(watch_proc, 0.5)
-            }
-        }
-        # }}}
-        if (echo) watch_proc()
-
         # TODO: exit time
         # just return the process
         res <- list(
@@ -788,6 +822,16 @@ eplus_run_wait <- function (proc, echo = TRUE) {
 eplus_exe <- function (eplus) {
     if (!is_avail_eplus(eplus)) use_eplus(eplus)
     config <- tryCatch(eplus_config(eplus), warning = function (w) stop(w))
+
+    if (config$version < 8.3) {
+        abort("error_eplus_lower_8.3", paste(
+            "Currently, eplusr only supports running IDFs of EnergyPlus v8.3 and above. ",
+            "This is because eplusr uses EnergyPlus command line interface ",
+            "which is available only in EnergyPlus v8.3 and above. ",
+            "You can update the version of your model using `version_updater()` and try again."
+        ))
+    }
+
     normalizePath(file.path(config$dir, config$exe), mustWork = TRUE)
 }
 # }}}
